@@ -33,7 +33,16 @@ from pythoncommons.project_utils import PROJECTS_BASEDIR_NAME, ProjectUtils
 from pythoncommons.string_utils import auto_str
 
 from yarndevtools.cdsw.constants import SECRET_PROJECTS_DIR
-from yarndevtools.commands_common import CommandAbs, EmailArguments
+from yarndevtools.commands_common import CommandAbs, EmailArguments, MongoArguments
+from yarndevtools.common.common_model import (
+    JobBuildData,
+    MONGO_COLLECTION_JENKINS_REPORTS,
+    FailedJenkinsBuild,
+    JobBuildDataStatus,
+    JenkinsTestcaseFilter,
+    JobBuildDataCounters,
+)
+from yarndevtools.common.db import MongoDbConfig, Database
 from yarndevtools.common.shared_command_utils import FullEmailConfig, CommandType
 from yarndevtools.constants import YARNDEVTOOLS_MODULE_NAME
 
@@ -94,121 +103,6 @@ class DownloadProgress:
         return f"{self.current_build_idx + 1}/{self.all_builds}"
 
 
-@dataclass
-class TestcaseFilter:
-    project_name: str
-    filter_expr: str
-
-    @property
-    def as_filter_spec(self):
-        return f"{self.project_name}:{self.filter_expr}"
-
-
-@auto_str
-class FailedJenkinsBuild:
-    def __init__(self, full_url_of_job: str, timestamp: int, job_name):
-        self.url = full_url_of_job
-        self.urls = JenkinsJobInstanceUrls(full_url_of_job)
-        self.build_number = full_url_of_job.rsplit("/")[-2]
-        self.timestamp = timestamp
-        self.job_name: str = job_name
-
-
-class JobBuildDataStatus(Enum):
-    # Invalid statuses
-    EMPTY = "Report does not contain testcase data"
-    NO_JSON_DATA_FOUND = "No JSON data found for build report"
-    CANNOT_FETCH = "Cannot fetch build report"
-    ALL_GREEN = "Build report contains tests but all are green"
-    # Valid statuses
-    HAVE_FAILED_TESTCASES = "Valid build report. Contains some failed tests"
-
-
-class JobBuildData:
-    def __init__(self, failed_build: FailedJenkinsBuild, counters, testcases, status: JobBuildDataStatus):
-        self._failed_build: FailedJenkinsBuild = failed_build
-        self.counters = counters
-        self.testcases: List[str] = testcases
-        self.filtered_testcases: List[FilteredResult] = []
-        self.filtered_testcases_by_expr: Dict[str, List[str]] = {}
-        self.no_of_failed_filtered_tc = None
-        self.unmatched_testcases: Set[str] = set()
-        self.status: JobBuildDataStatus = status
-        # TODO Save this to separate pickled object, so when JobBuildData's structure changes, we don't lose sent state for all jobs
-        self.mail_sent = False
-        self.sent_date = None
-
-    def has_failed_testcases(self):
-        return len(self.testcases) > 0
-
-    def filter_testcases(self, tc_filters: List[TestcaseFilter]):
-        matched_testcases = set()
-        for tcf in tc_filters:
-            matched_for_filter = list(filter(lambda tc: tcf.filter_expr in tc, self.testcases))
-            self.filtered_testcases.append(FilteredResult(tcf, matched_for_filter))
-            if tcf.filter_expr not in self.filtered_testcases_by_expr:
-                self.filtered_testcases_by_expr[tcf.filter_expr] = []
-            self.filtered_testcases_by_expr[tcf.filter_expr].extend(matched_for_filter)
-            matched_testcases.update(matched_for_filter)
-        self.no_of_failed_filtered_tc = sum([len(fr.testcases) for fr in self.filtered_testcases])
-        self.unmatched_testcases = set(self.testcases).difference(matched_testcases)
-
-    @property
-    def build_number(self):
-        return self._failed_build.build_number
-
-    @property
-    def build_url(self):
-        return self._failed_build.url
-
-    @property
-    def is_valid(self):
-        return self.status == JobBuildDataStatus.HAVE_FAILED_TESTCASES
-
-    @property
-    def is_mail_sent(self):
-        return self.mail_sent
-
-    @property
-    def tc_filters(self):
-        return [res.filter for res in self.filtered_testcases]
-
-    def __str__(self):
-        if self.is_valid:
-            return self._str_normal_report()
-        else:
-            return self._str_invalid_report()
-
-    def _str_invalid_report(self):
-        return (
-            f"Build number: {self.build_number}\n"
-            f"Build URL: {self.build_url}\n"
-            f"Invalid report! Details: {self.status.value}\n"
-        )
-
-    def _str_normal_report(self):
-        filtered_testcases: str = ""
-        if self.tc_filters:
-            for idx, ftcs in enumerate(self.filtered_testcases):
-                filtered_testcases += f"\nFILTER #{idx + 1}\n{str(ftcs)}\n"
-        if filtered_testcases:
-            filtered_testcases = f"\n{filtered_testcases}\n"
-
-        all_failed_testcases = "\n".join(self.testcases)
-        unmatched_testcases = "\n".join(self.unmatched_testcases)
-        return (
-            f"Counters:\n"
-            f"{self.counters}, "
-            f"Build number: {self.build_number}\n"
-            f"Build URL: {self.build_url}\n"
-            f"Matched testcases: {self.no_of_failed_filtered_tc}\n"
-            f"Unmatched testcases: {len(self.unmatched_testcases)}\n"
-            f"{filtered_testcases}\n"
-            f"Unmatched testcases:\n{unmatched_testcases}\n"
-            f"ALL Failed testcases:\n{all_failed_testcases}"
-        )
-
-
 class JenkinsJobUrls:
     def __init__(self, jenkins_base_url, job_name):
         self.jenkins_base_url = jenkins_base_url
@@ -221,21 +115,6 @@ class JenkinsJobUrls:
         return f"{jenkins_url}/job/{job_name}/api/json?tree=builds[url,result,timestamp]"
 
 
-class JenkinsJobInstanceUrls:
-    # Example URL: http://build.infra.cloudera.com/job/Mawo-UT-hadoop-CDPD-7.x/191/
-    def __init__(self, full_url):
-        self.full_url = full_url
-        self.job_console_output_url = self._append_to_url(full_url, "Console")
-        self.test_report_url = self._append_to_url(full_url, "testReport")
-        self.test_report_api_json_url = self.test_report_url + "/api/json?pretty=true"
-
-    @staticmethod
-    def _append_to_url(full_url, to_append):
-        if not full_url[-1] == "/":
-            full_url += "/"
-        return full_url + to_append
-
-
 class JenkinsApiConverter:
     @staticmethod
     def convert(job_name: str, jenkins_urls: JenkinsJobUrls, days: int):
@@ -267,7 +146,7 @@ class JenkinsApiConverter:
 
     @staticmethod
     def _list_builds(urls: JenkinsJobUrls) -> List[Any]:
-        """ List all builds of the target project. """
+        """List all builds of the target project."""
         url = urls.list_builds
         try:
             LOG.info("Fetching builds from Jenkins in url: %s", url)
@@ -351,20 +230,6 @@ class JenkinsApiConverter:
         return data
 
 
-@dataclass
-class FilteredResult:
-    filter: TestcaseFilter
-    testcases: List[str]
-
-    def __str__(self):
-        tcs = "\n".join(self.testcases)
-        s = f"Project: {self.filter.project_name}\n"
-        s += f"Filter expression: {self.filter.filter_expr}\n"
-        s += f"Number of failed testcases: {len(self.testcases)}\n"
-        s += f"Failed testcases (fully qualified name):\n{tcs}"
-        return s
-
-
 @auto_str
 class JenkinsJobReport:
     def __init__(self, job_build_datas, all_failing_tests, total_no_of_builds: int, num_builds_per_config: int):
@@ -384,6 +249,7 @@ class JenkinsJobReport:
         return self.actual_num_builds
 
     def __iter__(self):
+        self._index = 0
         return self
 
     def __next__(self):
@@ -436,16 +302,6 @@ class JenkinsJobReport:
             LOG.info(f"{self.all_failing_tests[tn]}: {tn}")
 
 
-@dataclass
-class JobBuildDataCounters:
-    failed: int
-    passed: int
-    skipped: int
-
-    def __str__(self):
-        return f"Failed: {self.failed}, Passed: {self.passed}, Skipped: {self.skipped}"
-
-
 @dataclass(frozen=True)
 class CachedBuildKey:
     job_name: str
@@ -691,8 +547,10 @@ class GoogleDriveCache(Cache):
 
 
 class CacheConfig:
-    def __init__(self, args, output_dir):
-        self.enabled: bool = not args.disable_file_cache if hasattr(args, "disable_file_cache") else False
+    def __init__(self, args, output_dir, force_download_mode=False):
+        self.enabled: bool = (
+            not args.disable_file_cache if hasattr(args, "disable_file_cache") and not force_download_mode else False
+        )
         self.reports_dir = FileUtils.ensure_dir_created(FileUtils.join_path(output_dir, "reports"))
         self.cached_data_dir = FileUtils.ensure_dir_created(FileUtils.join_path(output_dir, CACHED_DATA_DIRNAME))
         self.download_uncached_job_data: bool = (
@@ -779,7 +637,8 @@ class Email:
 class UnitTestResultFetcherConfig:
     def __init__(self, output_dir: str, args):
         self.args = args
-        self.cache: CacheConfig = CacheConfig(args, output_dir)
+        self.force_download_mode = args.force_download_mode if hasattr(args, "force_download_mode") else False
+        self.cache: CacheConfig = CacheConfig(args, output_dir, force_download_mode=self.force_download_mode)
         self.email: EmailConfig = EmailConfig(args)
         self.request_limit = args.req_limit if hasattr(args, "req_limit") and args.req_limit else 1
         self.jenkins_mode: UnitTestResultFetcherMode = (
@@ -791,9 +650,10 @@ class UnitTestResultFetcherConfig:
         self.job_names: List[str] = args.job_names.split(",")
         self.num_builds: int = self._determine_number_of_builds_to_examine(args.num_builds, self.request_limit)
         tc_filters_raw = args.tc_filters if hasattr(args, "tc_filters") and args.tc_filters else []
-        self.tc_filters: List[TestcaseFilter] = [TestcaseFilter(*tcf.split(":")) for tcf in tc_filters_raw]
+        self.tc_filters: List[JenkinsTestcaseFilter] = [
+            JenkinsTestcaseFilter(*tcf.split(":")) for tcf in tc_filters_raw
+        ]
         self.full_cmd: str = OsUtils.determine_full_command_filtered(filter_password=True)
-        self.force_download_mode = args.force_download_mode if hasattr(args, "force_download_mode") else False
         self.omit_job_summary: bool = args.omit_job_summary if hasattr(args, "omit_job_summary") else False
         self.fail_on_all_green_report: bool = False  # TODO hardcoded
         self.fail_on_empty_report: bool = False  # TODO hardcoded
@@ -801,6 +661,7 @@ class UnitTestResultFetcherConfig:
         self.reset_job_build_data_for_jobs: List[str] = (
             args.reset_job_build_data_for_jobs if hasattr(args, "reset_job_build_data_for_jobs") else []
         )
+        self.mongo_config = MongoDbConfig(args)
 
         # Validation
         if not self.tc_filters:
@@ -857,6 +718,7 @@ class UnitTestResultFetcher(CommandAbs):
         self.cache: Cache = self._create_cache(self.config)
         self.email: Email = Email(self.config.email)
         self.sent_requests: int = 0
+        self._database = UTResultFetcherDatabase(self.config.mongo_config)
 
     @staticmethod
     def create_parser(subparsers):
@@ -880,6 +742,7 @@ class UnitTestResultFetcher(CommandAbs):
             "--download-uncached-job-data",
         )
         EmailArguments.add_email_arguments(parser, add_subject=False, add_attachment_filename=False)
+        MongoArguments.add_mongo_arguments(parser)
 
         parser.add_argument(
             "--omit-job-summary",
@@ -1048,11 +911,12 @@ class UnitTestResultFetcher(CommandAbs):
             project_name=CommandType.UNIT_TEST_RESULT_FETCHER.output_dir_name,
             logger_name_prefix=YARNDEVTOOLS_MODULE_NAME,
             execution_mode=ExecutionMode.PRODUCTION,
-            console_debug=self.config.args.debug,
+            console_debug=self.config.args.logging_debug,
             postfix=None,
             repos=None,
             verbose_git_log=self.config.args.verbose,
             enable_logging_setup_debug_details=False,
+            with_trace_level=True,
         )
         if self.config.force_download_mode:
             LOG.info("FORCE DOWNLOAD MODE is on")
@@ -1102,6 +966,10 @@ class UnitTestResultFetcher(CommandAbs):
             self._process_build_data_from_report(build_data, report)
             self._print_report(build_data, report)
             self._invoke_report_processors(build_data, report)
+
+            key = list(report.jobs_by_url.keys())[0]
+            build_data = report.jobs_by_url[key]
+            self._database.save_build_data(build_data)
             # TODO fix
             # self._save_all_reports_to_cache(i, report)
 
@@ -1147,7 +1015,7 @@ class UnitTestResultFetcher(CommandAbs):
         self.cache.save_reports_meta(self.reports, log=log_report)
 
     def create_job_build_data(self, failed_build: FailedJenkinsBuild):
-        """ Find the names of any tests which failed in the given build output URL. """
+        """Find the names of any tests which failed in the given build output URL."""
         try:
             data = self.gather_raw_data_for_build(failed_build)
         except Exception:
@@ -1185,7 +1053,7 @@ class UnitTestResultFetcher(CommandAbs):
         return data
 
     def _create_jenkins_report(self, job_name: str) -> JenkinsJobReport:
-        """ Iterate runs of specified job within num_builds and collect results """
+        """Iterate runs of specified job within num_builds and collect results"""
         # TODO Discrepancy: request limit vs. days parameter
         jenkins_urls: JenkinsJobUrls = JenkinsJobUrls(self.config.jenkins_base_url, job_name)
         self.failed_builds, self.total_no_of_builds = JenkinsApiConverter.convert(
@@ -1240,8 +1108,20 @@ class UnitTestResultFetcher(CommandAbs):
         )
 
     @staticmethod
-    def _create_testcase_to_fail_count_dict(job_data, tc_to_fail_count):
+    def _create_testcase_to_fail_count_dict(job_data, tc_to_fail_count: Dict[str, int]):
         if job_data.has_failed_testcases():
             for failed_testcase in job_data.testcases:
                 LOG.info(f"Failed test: {failed_testcase}")
                 tc_to_fail_count[failed_testcase] = tc_to_fail_count.get(failed_testcase, 0) + 1
+
+
+class UTResultFetcherDatabase(Database):
+    def __init__(self, conf: MongoDbConfig):
+        super().__init__(conf)
+
+    def save_build_data(self, build_data: JobBuildData):
+        doc = super().find_by_id(build_data.build_url, collection_name=MONGO_COLLECTION_JENKINS_REPORTS)
+        if doc:
+            return doc
+
+        return super().save(build_data, collection_name=MONGO_COLLECTION_JENKINS_REPORTS, id_field_name="build_url")
